@@ -2,11 +2,12 @@
 
 """A node or step for the forcefield in a flowchart"""
 
-import forcefield_step
-import configargparse
 import logging
 import os.path
 import pkg_resources
+import pprint
+
+import forcefield_step
 import seamm_ff_util
 import seamm
 import seamm_util.printing as printing
@@ -17,58 +18,15 @@ job = printing.getPrinter()
 printer = printing.getPrinter('forcefield')
 
 
-def upcase(string):
-    """Return an uppercase version of the string.
-
-    Used for the type argument in argparse/
-    """
-    return string.upper()
-
-
 class Forcefield(seamm.Node):
 
     def __init__(self, flowchart=None, extension=None):
-        '''Initialize a forcefield step
+        """Initialize a forcefield step
 
         Keyword arguments:
-        '''
+        """
         logger.debug('Creating Forcefield {}'.format(self))
 
-        # Argument/config parsing
-        self.parser = configargparse.ArgParser(
-            auto_env_var_prefix='',
-            default_config_files=[
-                '/etc/seamm/forcefield_step.ini',
-                '/etc/seamm/seamm.ini',
-                '~/.seamm/forcefield_step.ini',
-                '~/.seamm/seamm.ini',
-            ]
-        )
-
-        self.parser.add_argument(
-            '--seamm-configfile',
-            is_config_file=True,
-            default=None,
-            help='a configuration file to override others'
-        )
-
-        # Options for this plugin
-        self.parser.add_argument(
-            "--forcefield-step-log-level",
-            default=configargparse.SUPPRESS,
-            choices=[
-                'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG', 'NOTSET'
-            ],
-            type=upcase,
-            help="the logging level for the Forcefield step"
-        )
-
-        self.options, self.unknown = self.parser.parse_known_args()
-
-        # Set the logging level for this module if requested
-        if 'forcefield_step_log_level' in self.options:
-            logger.setLevel(self.options.forcefield_step_log_level)
-
         super().__init__(
             flowchart=flowchart, title='Forcefield', extension=extension
         )
@@ -131,9 +89,6 @@ class Forcefield(seamm.Node):
 
         next_node = super().run(printer=printer)
 
-        # The options from command line, config file ...
-        o = self.options  # noqa: F841
-
         P = self.parameters.current_values_to_dict(
             context=seamm.flowchart_variables._data
         )
@@ -150,7 +105,8 @@ class Forcefield(seamm.Node):
         return next_node
 
     def assign_forcefield(self, P=None):
-        """Assign the forcefield to the structure, i.e. find the atom types.
+        """Assign the forcefield to the structure, i.e. find the atom types
+        and charges.
 
         Parameters
         ----------
@@ -161,39 +117,87 @@ class Forcefield(seamm.Node):
         -------
         None
         """
-        if P is None:
-            P = self.parameters.current_values_to_dict(
-                context=seamm.flowchart_variables._data
-            )
+        # if P is None:
+        #     P = self.parameters.current_values_to_dict(
+        #         context=seamm.flowchart_variables._data
+        #     )
 
         ff = self.get_variable('_forcefield')
-        system = self.get_variable('_system')
+        system_db = self.get_variable('_system_db')
+        configuration = system_db.system.configuration
 
         ffname = ff.current_forcefield
         printer.important(
             __(
-                f"Assigning the atom types for forcefield '{ffname}' to the "
-                "system",
+                "Assigning the atom types and charges for forcefield "
+                f"'{ffname}' to the system",
                 indent=self.indent + '    '
             )
         )
 
-        smiles = system.to_smiles(hydrogens=True)
+        # Atom types
+        logger.debug('Atom typing, getting the SMILES for the system')
+        smiles = configuration.to_smiles(hydrogens=True)
         logger.debug('Atom typing -- smiles = ' + smiles)
         ff_assigner = seamm_ff_util.FFAssigner(ff)
         atom_types = ff_assigner.assign(smiles, add_hydrogens=False)
         logger.info('Atom types: ' + ', '.join(atom_types))
-        key = f'atom_types_{ff.current_forcefield}'
-        if key not in system.atoms:
-            system.atoms.add_attribute(key, coltype='str')
-        system.atoms[key] = atom_types
+        key = f'atom_types_{ffname}'
+        if key not in configuration.atoms:
+            configuration.atoms.add_attribute(key, coltype='str')
+        configuration.atoms[key] = atom_types
 
-        printer.important(
-            __(
-                f"Assigned atom types to {system.n_atoms()} atoms.",
-                indent=self.indent + '    '
+        # Now get the charges if forcefield has them.
+        terms = ff.data['forcefield'][ffname]['parameters']
+        if 'bond_increments' in terms:
+            logger.debug('Getting the charges for the system')
+            neighbors = configuration.bonded_neighbors(as_indices=True)
+
+            charges = []
+            total_q = 0.0
+            for i in range(configuration.n_atoms):
+                itype = atom_types[i]
+                parameters = ff.charges(itype)[3]
+                q = float(parameters['Q'])
+                for j in neighbors[i]:
+                    jtype = atom_types[j]
+                    parameters = ff.bond_increments(itype, jtype)[3]
+                    q += float(parameters['deltaij'])
+                charges.append(q)
+                total_q += q
+            if abs(total_q) > 0.0001:
+                logger.warning('Total charge is not zero: {}'.format(total_q))
+                logger.info(
+                    'Charges from increments and charges:\n' +
+                    pprint.pformat(charges)
+                )
+            else:
+                logger.debug(
+                    'Charges from increments:\n' + pprint.pformat(charges)
+                )
+
+            key = f'charges_{ffname}'
+            if key not in configuration.atoms:
+                configuration.atoms.add_attribute(key, coltype='float')
+            charge_column = configuration.atoms.get_column(key)
+            charge_column[0:] = charges
+            logger.debug(f"Set column '{key}' to the charges")
+
+            printer.important(
+                __(
+                    "Assigned atom types and charges to "
+                    f"{configuration.n_atoms} atoms.",
+                    indent=self.indent + '    '
+                )
+            )
+        else:
+            printer.important(
+                __(
+                    f"Assigned atom types to {configuration.n_atoms} "
+                    "atoms.",
+                    indent=self.indent + '    '
+                )
             )
-        )
 
     def setup_forcefield(self, P=None):
         """Setup the forcefield for later use.
@@ -235,9 +239,10 @@ class Forcefield(seamm.Node):
             path = pkg_resources.resource_filename(__name__, 'data/')
             ff_file = os.path.join(path, P['forcefield_file'])
 
+            ff = seamm_ff_util.Forcefield(ff_file)
+            self.set_variable('_forcefield', ff)
+
             if P['forcefield'] == 'default':
-                ff = seamm_ff_util.Forcefield(ff_file)
-                self.set_variable('_forcefield', ff)
                 printer.important(
                     __(
                         "   Using the default forcefield '{ff}'.",
@@ -245,9 +250,9 @@ class Forcefield(seamm.Node):
                         indent=self.indent + '    '
                     )
                 )
+
+                ff.initialize_biosym_forcefield()
             else:
-                ff = seamm_ff_util.Forcefield(ff_file, P['forcefield'])
-                self.set_variable('_forcefield', ff)
                 printer.important(
                     __(
                         "   Using the forcefield '{forcefield}'",
@@ -256,4 +261,4 @@ class Forcefield(seamm.Node):
                     )
                 )
 
-            ff.initialize_biosym_forcefield()
+                ff.initialize_biosym_forcefield(P['forcefield'])
